/*
 *
 * Author Jean-Pierre PARISY <jpparisy@free.fr>
 *
 * Sequencing flash depending on pulse variation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include <htc.h>
#include <pic.h>
#include <pic12f509.h>
#define _XTAL_FREQ  4000000     			// oscillator frequency for _delay()

 __CONFIG(MCLRE_OFF & CP_OFF & WDT_OFF & OSC_IntRC);

unsigned char cnt, tempo, sequence;

/* ************************************************************************* */
main (void)
{
	OPTION = ~T0CS & ~PSA & (nGPWU | nGPPU | T0SE | 0x2);	/*
	internal cycle clock
	prescaler assigned to timer 
	wake up disabled
	pull up disabled
	high to low source edge
	prescaler values:	+-----+-------+
						| 0x0 | 1:2   |
						| 0x1 | 1:4   |
						| 0x2 | 1:8   | <---
						| 0x3 | 1:16  |
						| 0x4 | 1:32  |
						| 0x5 | 1:64  |
						| 0x6 | 1:128 |
						| 0x7 | 1:256 |
						+-----+-------+	*/
	TRIS = 0b001000;						// GP3 as input, all other as output
	GPIO = 0;								// all ports off
	sequence = tempo = 0;
	for (cnt = 0; cnt < 5; cnt++) {			// wait for five pulses
		do {} while (!GP3);					// wait start of pulse
		do {} while (GP3);					// wait end of pulse
	}
/* ************************************************************************* */
	for (;;) {								// loop for ever
		do {								// wait pulse
		} while (!GP3);						// pulse received

		TMR0 = 255 - 106;					// wait at least 106 * 8 = 848 usec
		do {} while (TMR0 >= 20);
		do {} while (GP3);					// wait end of pulse
		cnt = TMR0 + 1;						// TMR0 is now between 16 and 144, neutral is 80
		cnt = (cnt - 14) >> 1;				// cnt between 1 :: 64
		tempo--;
//           1         2         3         4         5         6
// 01234567890123456789012345678901234567890123456789012345678901234
// | < fast --------------- slow > | < slow --------------- fast > |
//
		if (tempo > 32) {
			if (cnt > 32)					// 32 = slow, 0 = fast
				tempo = 64 - cnt;
			else
				tempo = cnt;
			switch (sequence) {			// set ports according to the sequence number
			case 0:
				GPIO = 0b000001;		// set port 0
				break;
			case 1:
				GPIO = 0b000010;		// set port 1
				break;
			case 2:
				GPIO = 0b000100;		// set port 2
				break;
			case 3:
				GPIO = 0b010000;		// set port 4
				break;
			case 4:
				GPIO = 0b100000;		// set port 5
				break;
			}
			if (++sequence > 4)
				sequence = 0;
		}
	}
}
